"""Various misc. utilities."""

from typing import (
    TypeVar,
    Callable,
    Iterable,
)

from collections.abc import (
    Sequence,
)

# from logging import LoggerAdapter
# import copy


T = TypeVar('T')
U = TypeVar('U')


def group_by(proc: Callable[[T], U], seq: Sequence[T]) -> dict[U, list[T]]:
    """
    Group elements in seq by proc.

    Return a dictionary mapping the result of proc onto lists of each
    element which evaluated to that key.
    """
    d: dict[U, list[T]] = {}
    for item in seq:
        key = proc(item)
        d[key] = (d.get(key) or []) + [item]
    return d


def concatenate(lstlst: Iterable[Iterable[T]]) -> list[T]:
    """Concatenate a list of lists into a flat(er) list."""
    out: list[T] = []
    for lst in lstlst:
        out += lst
    return out


# class ILoggerAdapter(LoggerAdapter):
#     """Wrapper around a logger, allowing extra keys to added once."""
#
#     def __init__(self, logger, extra):
#         super().__init__(logger, extra)
#         self.env = extra
#
#     def process(self, msg: str, kwargs):
#         """Overridden method from upstream."""
#         msg, kwargs = super().process(msg, kwargs)
#         result = copy.deepcopy(kwargs)
#         default_kwargs_key = ['exc_info', 'stack_info', 'extra']
#         custom_key = [k for k in result.keys() if k not in default_kwargs_key]
#         result['extra'].update({k: result.pop(k) for k in custom_key})
#
#         return msg, result
